@@ -817,20 +817,16 @@ dnf_repo_set_metadata_expire(DnfRepo *repo, guint metadata_expire)
 /**
 * @brief Format user password string
 *
-* Returns user and password in user:password form. If encode is True,
-* special characters in user and password are URL encoded.
+* Returns user and password in user:password form.
+* Special characters in user and password are URL encoded.
 *
 * @param user Username
 * @param passwd Password
-* @param encode If quote is True, special characters in user and password are URL encoded.
 * @return User and password in user:password form
 */
-static std::string formatUserPassString(const std::string & user, const std::string & passwd, bool encode)
+static std::string formatUserPassString(const std::string & user, const std::string & passwd)
 {
-    if (encode)
-        return libdnf::urlEncode(user) + ":" + libdnf::urlEncode(passwd);
-    else
-        return user + ":" + passwd;
+    return libdnf::urlEncode(user) + ":" + libdnf::urlEncode(passwd);
 }
 
 /* Resets repository configuration options previously readed from repository
@@ -969,6 +965,9 @@ dnf_repo_set_keyfile_data(DnfRepo *repo, gboolean reloadFromGKeyFile, GError **e
         dnf_repo_apply_setopts(*conf, repoId);
     }
 
+    if (dnf_context_get_cache_dir(priv->context))
+        conf->basecachedir().set(libdnf::Option::Priority::REPOCONFIG, dnf_context_get_cache_dir(priv->context));
+
     /* baseurl is optional; if missing, unset it */
     g_auto(GStrv) baseurls = NULL;
     auto & repoBaseurls = conf->baseurl().getValue();
@@ -1127,7 +1126,7 @@ dnf_repo_set_keyfile_data(DnfRepo *repo, gboolean reloadFromGKeyFile, GError **e
                             "repo '%s': 'proxy_username' is set but not 'proxy_password'", repoId);
                 return FALSE;
             }
-            tmp_str = formatUserPassString(tmp_str, conf->proxy_password().getValue(), true);
+            tmp_str = formatUserPassString(tmp_str, conf->proxy_password().getValue());
             tmp_cstr = tmp_str.c_str();
         }
     }
@@ -1135,14 +1134,11 @@ dnf_repo_set_keyfile_data(DnfRepo *repo, gboolean reloadFromGKeyFile, GError **e
         return FALSE;
 
     // setup username and password
-    tmp_cstr = NULL;
-    tmp_str = conf->username().getValue();
-    if (!tmp_str.empty()) {
-        // TODO Use URL encoded form, needs support in librepo
-        tmp_str = formatUserPassString(tmp_str, conf->password().getValue(), false);
-        tmp_cstr = tmp_str.c_str();
-    }
-    if (!lr_handle_setopt(priv->repo_handle, error, LRO_USERPWD, tmp_cstr))
+    auto & username = conf->username().getValue();
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_USERNAME, username.empty() ? NULL : username.c_str()))
+        return FALSE;
+    auto & password = conf->password().getValue();
+    if (!lr_handle_setopt(priv->repo_handle, error, LRO_PASSWORD, password.empty() ? NULL : password.c_str()))
         return FALSE;
 
     auto proxy_sslverify = conf->proxy_sslverify().getValue();
@@ -1463,6 +1459,7 @@ dnf_repo_check_internal(DnfRepo *repo,
                     error_local->message);
         return FALSE;
     }
+    repoImpl->repomdFn = yum_repo->repomd;
 
     /* get timestamp */
     ret = lr_result_getinfo(priv->repo_result, &error_local,
@@ -1765,8 +1762,10 @@ dnf_repo_update(DnfRepo *repo,
     if (!dnf_repo_set_keyfile_data(repo, TRUE, error))
         return FALSE;
 
+    auto repoImpl = libdnf::repoGetImpl(priv->repo);
+
     /* countme support */
-    libdnf::repoGetImpl(priv->repo)->addCountmeFlag(priv->repo_handle);
+    repoImpl->addCountmeFlag(priv->repo_handle);
 
     /* take lock */
     ret = dnf_state_take_lock(state,
@@ -1784,6 +1783,28 @@ dnf_repo_update(DnfRepo *repo,
     if (!ret)
         goto out;
 
+    state_local = dnf_state_get_child(state);
+    dnf_state_action_start(state_local,
+                           DNF_STATE_ACTION_DOWNLOAD_METADATA, NULL);
+
+    try {
+        if (repoImpl->isInSync()) {
+            /* reset timestamp */
+            ret = utimes(repoImpl->repomdFn.c_str(), NULL);
+            if (ret != 0)
+                g_debug("Failed to reset timestamp on repomd.xml");
+            ret = dnf_state_done(state, error);
+            if (!ret)
+                goto out;
+
+            /* skip check */
+            ret = dnf_state_finished(state, error);
+            goto out;
+        }
+    } catch (std::exception & ex) {
+        g_debug("Failed to verify if metadata is in sync: \"%s\". Proceding with download", ex.what());
+    }
+
     /* remove the temporary space if it already exists */
     if (g_file_test(priv->location_tmp, G_FILE_TEST_EXISTS)) {
         ret = dnf_remove_recursive(priv->location_tmp, error);
@@ -1842,7 +1863,7 @@ dnf_repo_update(DnfRepo *repo,
         goto out;
 
     /* Callback to display progress of downloading */
-    state_local = updatedata.state = dnf_state_get_child(state);
+    updatedata.state = state_local;
     ret = lr_handle_setopt(priv->repo_handle, error,
                            LRO_PROGRESSDATA, &updatedata);
     if (!ret)
@@ -1867,8 +1888,6 @@ dnf_repo_update(DnfRepo *repo,
     }
 
     lr_result_clear(priv->repo_result);
-    dnf_state_action_start(state_local,
-                           DNF_STATE_ACTION_DOWNLOAD_METADATA, NULL);
     ret = lr_handle_perform(priv->repo_handle,
                             priv->repo_result,
                             &error_local);
